
/* ************************************************************************ *\

Copyright (c) 2011, Atmel Corporation All rights reserved. 

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

3. The name of ATMEL may not be used to endorse or promote products derived 
from this software without specific prior written permission.  

THIS SOFTWARE IS PROVIDED BY ATMEL ``AS IS'' AND ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND 
SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

//#include <avr/io.h> //for debug
#include <avr/interrupt.h>
#include <util/delay.h>
#include "TWI.h"

#define SCLK (1<<PB2)
#define RCLK (1<<PB4)
#define DIO  (1<<PB3)

uint8_t reg=0;

static void spi_transmit(uint8_t data) {
    uint8_t i;

    for (i=0; i<8; i++)
    {
        PORTB=(data & 0x80) ? PORTB | DIO : PORTB & ~DIO;

        data=(data<<1);

        PORTB |= SCLK;
        PORTB &= ~SCLK;
    }
}

static void to_led(uint8_t value, uint8_t reg) {
static uint8_t digit[10] = {
      0b11000000, // 0
      0b11111001, // 1
      0b10100100, // 2
      0b10110000, // 3
      0b10011001, // 4
      0b10010010, // 5
      0b10000010, // 6
      0b11111000, // 7
      0b10000000, // 8
      0b10010000, // 9
};
    PORTB &= ~RCLK;
    spi_transmit(digit[value%10]);
    spi_transmit(1<<reg);
    PORTB |= RCLK;

}

void show_led(uint16_t num) {
        switch (reg) {
        case 0:
            to_led((uint8_t)(num%10),0);
            break;
        case 1:
            if (num>=10)
                to_led((uint8_t)((num%100)/10),1);
            break;
        case 2:
            if (num>=100)
                to_led((uint8_t)(num/100),2);
            break;
        }

        reg=(reg ==  2) ? 0 : reg+1;
}

#define InvBit(reg, bit)   reg ^= (1<<(bit))
#define ClearBit(reg, bit)       reg &= (~(1<<(bit)))
#define SetBit(reg, bit)          reg |= (1<<(bit)) 
#define BitIsSet(reg, bit)       ((reg & (1<<bit)) != 0)


int main() {
	CLKPR =(1<<CLKPCE);
    CLKPR =0; // set 9.6 MHz CPU freq
	DDRB |= (SCLK | RCLK | DIO);

	sei();   //!< Enable global interrupts.

	twi_slave_init();
	twi_slave_enable();

  	//SetBit(DDRB, DDB4); //for debug
	uint16_t count;
  	for(;;) {
		if (reg==0) count=incomingBuffer[1]+(uint16_t)((incomingBuffer[2])<<8);
		show_led(count);
        _delay_ms(5);
	}
}
